/* vim: set sts=2 sw=2 et :
 *
 * This program does file/directory metadata changes for the grilo monitor test
 *
 */

#include <gio/gio.h>
#include <glib.h>

#include "grilo-do-metadata-changes.h"

#define DATA "qwertyuiop"

static GFile*
create_file_for_test (gchar *path)
{
  GFile *file;
  GError *error = NULL;
  GFileCreateFlags flags = G_FILE_CREATE_REPLACE_DESTINATION;
  GriloTestAction *add_action = g_new0 (GriloTestAction, 1);

  g_debug ("Is stack stale?");
  while (stack_fresh)
    g_thread_yield ();
  g_debug ("Stack is stale! Creating %s", path);

  file = g_file_new_for_path (path);
  g_file_create (file, flags, NULL, &error);
  if (error)
    g_error ("Unable to create file %s: %s", path, error->message);

  add_action->change_type = GRL_CONTENT_ADDED;
  add_action->change_path = g_path_get_dirname (path);
  g_mutex_lock (&mutex);
  actions = g_slist_append (actions, add_action);
  stack_fresh = TRUE;
  g_mutex_unlock (&mutex);

  g_free (path);
  return file;
}

static GFile*
create_directory_for_test (gchar *path)
{
  GFile *dir;
  GError *error = NULL;
  GriloTestAction *add_action = g_new0 (GriloTestAction, 1);

  g_debug ("Is stack stale?");
  while (stack_fresh)
    g_thread_yield ();
  g_debug ("Stack is stale! Creating %s", path);

  dir = g_file_new_for_path (path);
  g_file_make_directory (dir, NULL, &error);
  if (error)
    g_error ("Unable to create dir %s: %s", path, error->message);

  add_action->change_type = GRL_CONTENT_ADDED;
  add_action->change_path = g_path_get_dirname (path);
  g_mutex_lock (&mutex);
  actions = g_slist_append (actions, add_action);
  stack_fresh = TRUE;
  g_mutex_unlock (&mutex);

  g_free (path);
  return dir;
}

static void
delete_gfile_for_test (GFile    *file,
                       gboolean  is_file)
{
  gchar *path;
  GError *error = NULL;
  GriloTestAction *remove_action = g_new0 (GriloTestAction, 1);
  path = g_file_get_parse_name (file);

  g_debug ("Is stack stale?");
  while (stack_fresh)
    g_thread_yield ();
  g_debug ("Stack is stale! Deleting %s", path);

  g_file_delete (file, NULL, &error);
  if (error)
    g_error ("Unable to delete %s: %s", path, error->message);

  remove_action->change_type = GRL_CONTENT_REMOVED;
  remove_action->change_path = g_path_get_dirname (path);
  g_mutex_lock (&mutex);
  actions = g_slist_append (actions, remove_action);
  if (is_file)
    actions = g_slist_append (actions, g_slice_dup (GriloTestAction, remove_action));
  stack_fresh = TRUE;
  g_mutex_unlock (&mutex);

  g_free (path);
  g_object_unref (file);
}

static void
write_file_for_test (GFile *file)
{
  gchar *path;
  GError *error = NULL;
  GOutputStream *file_writer;
  GriloTestAction *action = g_new0 (GriloTestAction, 1);

  path = g_file_get_parse_name (file);
  g_debug ("Is stack stale?");
  while (stack_fresh)
    g_thread_yield ();
  g_debug ("Stack is stale! Writing to file: %s", path);

  file_writer = g_io_stream_get_output_stream (G_IO_STREAM (g_file_open_readwrite (file, NULL, &error)));
  if (error)
    g_error ("Unable to open file %s: %s", path, error->message);
  g_output_stream_write (file_writer, DATA, sizeof(DATA), NULL, &error);
  if (error)
    g_error ("Unable to write to file %s: %s", path, error->message);
  g_output_stream_close (file_writer, NULL, NULL);

  action->change_type = GRL_CONTENT_CHANGED;
  action->change_path = g_path_get_dirname (path);
  g_mutex_lock (&mutex);
  actions = g_slist_append (actions, action);
  stack_fresh = TRUE;
  g_mutex_unlock (&mutex);

  g_free (path);
}

/* Begin testing. Given a directory being monitored recursively, steps are:
 * > Create file
 * > Create sub-directory
 * > Create file inside sub-directory
 * > Create directory inside sub-directory
 * > Write to file
 * > Remove directory inside sub-directory
 * > Remove file inside sub-directory
 * > Remove sub-directory
 * > Remove file */
static void
begin_monitor_testing (void)
{
  GFile *file, *subdir, *subdir_file, *subdir_dir;
  gchar *tmp;
  GError *error = NULL;


  g_printf ("Running test in: %s\n", toplevel_path);
  /* Create files and directories */
  g_printf ("Testing notifications of created files ...\n");
  g_usleep (2*G_USEC_PER_SEC); /* Workaround races */
  tmp = g_build_filename (toplevel_path, "file", NULL);
  file = create_file_for_test (tmp);

  g_usleep (2*G_USEC_PER_SEC); /* Workaround races */
  tmp = g_build_filename (toplevel_path, "subdir", NULL);
  subdir = create_directory_for_test (tmp);

  g_usleep (2*G_USEC_PER_SEC); /* Workaround races */
  tmp = g_build_filename (toplevel_path, "subdir", "file", NULL);
  subdir_file = create_file_for_test (tmp);

  g_usleep (2*G_USEC_PER_SEC); /* Workaround races */
  tmp = g_build_filename (toplevel_path, "subdir", "subdir", NULL);
  subdir_dir = create_directory_for_test (tmp);

  /* Write to file */
  g_printf ("Testing notifications of changed files ...\n");
  write_file_for_test (file);

  /* Remove files and directories */
  g_printf ("Testing notifications of removed files ...\n");
  delete_gfile_for_test (subdir_dir, 0);
  delete_gfile_for_test (subdir_file, 1);
  delete_gfile_for_test (subdir, 0);
  delete_gfile_for_test (file, 1);

  g_printf ("Success!\n");
  g_printf ("(Ignore any warnings by Grilo about non-existent files. It's fine.)\n");

  g_main_loop_quit (loop);
}
